<?php
/**
 * Bootstrapper. Initializes the plugin.
 *
 * @package    MPP_Vimeo_Integration
 * @subpackage Bootstrap
 * @copyright  Copyright (c) 2018, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Brajesh Singh
 * @since      1.0.0
 */

namespace MPP_Vimeo_Integration\Bootstrap;

use MPP_Vimeo_Integration\Admin\MPP_Vimeo_Admin_Settings;
use MPP_Vimeo_Integration\Handlers\MPP_Vimeo_Actions_Handler;
use Vimeo\Exceptions\VimeoRequestException;
use Vimeo\Vimeo;

// No direct access.
if ( ! defined( 'ABSPATH' ) ) {
	exit( 0 );
}

/**
 * Bootstrapper.
 */
class MPP_Vimeo_Bootstrapper {

	use MPP_Vimeo_Bootable;

	/**
	 * Bind hooks
	 */
	private function setup() {
		add_action( 'plugins_loaded', array( $this, 'load' ), 1 );
		add_action( 'plugins_loaded', array( $this, 'init' ), 2 );
		add_action( 'pp_vimeo_hourly_check', array( $this, 'process_videos' ) );
		add_action( 'init', array( $this, 'load_translations' ) );
	}

	/**
	 * Load core functions/template tags.
	 * These are non auto loadable constructs.
	 */
	public function load() {

		if ( ! $this->has_min_php_version() ) {
			return;
		}

		$path = mpp_vimeo()->path;

		$files = array(
			'src/core/mpp-vimeo-functions.php',
		);

		foreach ( $files as $file ) {
			require_once $path . $file;
		}
	}

	/**
	 * Boot classes
	 */
	public function init() {

		if ( ! $this->has_min_php_version() ) {
			return;
		}

		MPP_Vimeo_Actions_Handler::boot();

		if ( is_admin() && ! defined( 'WP_DOING_AJAX' ) ) {
			MPP_Vimeo_Admin_Settings::boot();
		}
	}

	/**
     * Process video
     *
	 * @param int $media_id Media id.
	 * @param int $vimeo_video_id Vimeo video id.
	 */
	public function process_video( $media_id, $vimeo_video_id ) {
		$client_id     = mpp_get_option( 'mpp_vimeo_client_id', '' );
		$client_secret = mpp_get_option( 'mpp_vimeo_client_secret', '' );
		$client_token  = mpp_get_option( 'mpp_vimeo_access_token', '' );

		if ( empty( $client_id ) || empty( $client_secret || $client_token ) ) {
			return;
		}

		$lib = new Vimeo( $client_id, $client_secret, $client_token );

		try {
		    $response = $lib->request( '/videos/' . $vimeo_video_id, array(), 'GET' );
        } catch ( VimeoRequestException $e ) {
		    error_log( '[vimeo error]' . $e->getMessage() );
        }

        if ( empty( $response['body'] ) || empty( $response['body']['transcode'] )
        || 'complete' != $response['body']['transcode']['status'] ) {
            return;
        }

        $file = mpp_get_media_meta( $media_id, '_video_path', true );

        if ( is_readable( $file ) ) {
            unlink( $file );
            mpp_delete_media_meta( $media_id, '_video_path' );
            mpp_delete_media_meta( $media_id, '_vimeo_video_id' );
        }
    }

	/**
	 * Process videos
	 */
	public function process_videos() {
        global $wpdb;

        $items = $wpdb->get_results( $wpdb->prepare( "SELECT * From {$wpdb->postmeta} WHERE meta_key = %s", '_vimeo_video_id' ) );

        if ( empty( $items ) ) {
            return;
        }

        foreach ( $items as $item ) {

            if ( empty( $item['post_id'] ) || empty( $item['meta_value'] ) ) {
                continue;
            }

            $this->process_video( $item['post_id'], $item['meta_value'] );
        }
    }

	/**
	 * Load translations.
	 */
	public function load_translations() {
		load_plugin_textdomain( 'mpp-vimeo-integration', false, basename( dirname( mpp_vimeo()->path ) ) . '/languages' );
	}

	/**
	 * Version check
	 *
	 * @return bool
	 */
	public function has_min_php_version() {

		if ( version_compare( PHP_VERSION, mpp_vimeo()->php_version, '<' ) ) {
			add_action( 'admin_notices', array( $this, 'render_notice' ) );

			return false;
		};

		return true;
	}

	/**
	 * Render notice
	 */
	private function render_notice() {
		?>
		<div class="notice notice-error is_dismissable">
			<p>
				<?php esc_html_e( 'You need to update your PHP version to run this plugin.', 'mpp-vimeo-integration' ); ?> <br />
				<?php
				printf(
					esc_html__( 'Actual version is: %1$s, required version is: %2$s.', 'mpp-vimeo-integration' ),
					PHP_VERSION,
					mpp_vimeo()->php_version
				); // phpcs: XSS ok.
				?>
			</p>
		</div>
		<?php
	}
}
