<?php
/**
 * Action handler class
 *
 * @package MPP_Vimeo_Integration
 * @subpackage Handlers
 */

namespace MPP_Vimeo_Integration\Handlers;

use MPP_Vimeo_Integration\Bootstrap\MPP_Vimeo_Bootable;
use Vimeo\Exceptions\VimeoRequestException;
use Vimeo\Exceptions\VimeoUploadException;
use Vimeo\Vimeo;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Actions_Handler
 */
class MPP_Vimeo_Actions_Handler {

	use MPP_Vimeo_Bootable;

	/**
	 * Setup
	 */
	private function setup() {
		add_action( 'mpp_media_added', array( $this, 'on_add' ) );
	}

	/**
	 * On media add
	 *
	 * @param int $media_id Media id.
	 */
	public function on_add( $media_id ) {
		$media = mpp_get_media( $media_id );

		if ( 'video' !== $media->type ) {
			return;
		}

		$client_id     = mpp_get_option( 'mpp_vimeo_client_id', '' );
		$client_secret = mpp_get_option( 'mpp_vimeo_client_secret', '' );
		$client_token  = mpp_get_option( 'mpp_vimeo_access_token', '' );

		if ( empty( $client_id ) || empty( $client_secret || $client_token ) ) {
			return;
		}

		$lib = new Vimeo( $client_id, $client_secret, $client_token );

		mpp_update_media_meta( $media_id, '_mpp_vimeo_proccessing', 1 );

		$media_src  = mpp_get_media_src( 'original', $media_id );

		try {
			$response = $lib->request(
				'/me/videos',
				array(
					'upload'  => array(
						'approach' => 'pull',
						'link'     => $media_src,
					),
					'name'    => $media->title,
					'privacy' => array(
						'view' => 'anybody',
					),
				),
				'POST'
			);
		} catch ( VimeoUploadException $e ) {
			error_log( 'mpp vimeo error: ' . $e->getMessage() );
			return;
		} catch ( VimeoRequestException $e ) {
			error_log( 'mpp vimeo error: ' . $e->getMessage() );
			return;
		}

		if ( empty( $response['body'] ) || empty( $response['body']['embed'] ) ) {
			return;
		}

		$context       = mpp_get_media_meta( $media_id, '_mpp_context', true );
		$local_storage = mpp_get_storage_manager();

		$file = $local_storage->get_path( 'original', $media_id );

		$video_id = trim( str_replace( '/videos/', '', $response['body']['uri'] ) );

		mpp_delete_media_meta( $media_id );
		mpp_update_media_meta( $media_id, '_vimeo_video_id', $video_id );
		mpp_update_media_meta( $media_id, '_video_path', $file );
		mpp_update_media_meta( $media_id, '_mpp_component_id', $media->component_id );
		mpp_update_media_meta( $media_id, '_mpp_context', $context );
		mpp_update_media_meta( $media_id, '_mpp_is_oembed', 1 );
		mpp_update_media_meta( $media_id, '_mpp_source', $response['body']['link'] );
		mpp_update_media_meta( $media_id, '_mpp_oembed_content', $response['body']['embed']['html'] );
		mpp_update_media_meta( $media_id, '_mpp_oembed_time', time() );
	}

	/**
	 * Get token
	 *
	 * @param Vimeo $lib Vimeo lib object.
	 *
	 * @return string|null
	 */
	private function get_token( $lib ) {
		$token = get_transient( 'mpp_vimeo_access_token' );

		if ( ! empty( $token ) ) {
			return $token;
		}

		$token = $lib->clientCredentials( 'upload' );

		if ( ! empty( $token['body'] ) && ! empty( $token['body']['access_token'] ) ) {
			set_transient( 'mpp_vimeo_access_token', $token['body']['access_token'], '86400' );

			return $token['body']['access_token'];
		}

		return null;
	}
}

